
from BaseDataMaintenance.common.milvusUtil import *
from multiprocessing import Process,Queue
from BaseDataMaintenance.common.Utils import *
from BaseDataMaintenance.common.multiThread import MultiThreadHandler
from apscheduler.schedulers.blocking import BlockingScheduler

from BaseDataMaintenance.model.ots.document_product_dict import *

from BaseDataMaintenance.dataSource.source import getConnect_ots

from tablestore import *
from BaseDataMaintenance.common.Utils import getRow_ots

from BaseDataMaintenance.maintenance.product.productUtils import *

import time
import traceback
import json
import requests



class Product_Dict_Manager():

    def __init__(self):

        self.collection_name_name = COLLECTION_NAME_NAME
        self.collection_name_brand = COLLECTION_NAME_BRAND
        self.collection_name_specs = COLLECTION_NAME_SPECS
        self.search_params = {"metric_type":"IP",
                              "params":{"nprobe":10}}

        self.init_milvus()
        self.ots_client = getConnect_ots()
        self.queue_product_dict = Queue()

        self.session = requests.Session()


        self.Coll_name = getCollection(self.collection_name_name)
        self.Coll_brand = getCollection(self.collection_name_brand)
        self.Coll_specs = getCollection(self.collection_name_specs)


    def init_milvus(self):
        from pymilvus import connections,FieldSchema,DataType
        fields = [
            # FieldSchema(name="pk_id",dtype=DataType.INT64,is_primary=True,auto_id=True), # pk is the same as ots
            FieldSchema(name="ots_id",dtype=DataType.VARCHAR,max_length=32,is_primary=True),
            FieldSchema(name="ots_name",dtype=DataType.VARCHAR,max_length=MAX_NAME_LENGTH),
            FieldSchema(name="standard_name",dtype=DataType.VARCHAR,max_length=MAX_NAME_LENGTH),
            FieldSchema(name="standard_name_id",dtype=DataType.VARCHAR,max_length=32),
            FieldSchema(name="embedding",dtype=DataType.FLOAT_VECTOR,dim=1024),
            FieldSchema(name="ots_parent_id",dtype=DataType.VARCHAR,max_length=32),
            FieldSchema(name="ots_grade",dtype=DataType.INT64)
        ]

        index_name = "embedding"
        index_params = {"params":{"nlist":2048},
                        "index_type":"IVF_SQ8",
                        "metric_type":"IP"}

        init_milvus(milvus_host)

        #build the product name brand specs embedding respectively
        create_embedding_schema(self.collection_name_name,fields,index_name,index_params)
        create_embedding_schema(self.collection_name_brand,fields,index_name,index_params)
        create_embedding_schema(self.collection_name_specs,fields,index_name,index_params)






    def embedding_producer(self,columns=[DOCUMENT_PRODUCT_DICT_NAME,DOCUMENT_PRODUCT_DICT_PARENT_ID,DOCUMENT_PRODUCT_DICT_GRADE,DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS]):

        bool_query = BoolQuery(
            must_queries=[RangeQuery(DOCUMENT_PRODUCT_DICT_GRADE,3,5,True,True)],
            must_not_queries=[TermQuery(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED,2)])

        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED)]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))

        list_dict = getRow_ots(rows)
        for _d in list_dict:
            self.queue_product_dict.put(_d)

        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))
            list_dict = getRow_ots(rows)
            for _d in list_dict:
                self.queue_product_dict.put(_d)
            if self.queue_product_dict.qsize()>=1000:
                break
        log("product_dict embedding total_count:%d"%total_count)



    def get_collection(self,grade):
        Coll = None
        Coll_name = None
        if grade ==SPECS_GRADE:
            Coll = self.Coll_specs
            Coll_name = self.collection_name_specs
        if grade == BRAND_GRADE:
            Coll = self.Coll_brand
            Coll_name = self.collection_name_brand
        if grade == NAME_GRADE:
            Coll = self.Coll_name
            Coll_name = self.collection_name_name
        return Coll,Coll_name

    def embedding_comsumer(self):
        def handle(item,result_queue):
            try:
                id = item.get(DOCUMENT_PRODUCT_DICT_ID)
                name = str(item.get(DOCUMENT_PRODUCT_DICT_NAME))[:MAX_NAME_LENGTH]
                vector = request_embedding(name)
                parent_id = item.get(DOCUMENT_PRODUCT_DICT_PARENT_ID)
                grade = item.get(DOCUMENT_PRODUCT_DICT_GRADE)
                Coll,_ = self.get_collection(grade)
                standard_alias = item.get(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS,"")
                if vector is not None and Coll is not None:

                    data = [[id],
                            [name],
                            [name],
                            [id],
                            [vector],
                            [parent_id],
                            [grade]]
                    insert_embedding(Coll,data)


                    if standard_alias is not None and standard_alias!="":
                        list_alias = standard_alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR)
                        for _alias in list_alias:
                            _alias = _alias.strip()
                            if len(_alias)==0:
                                continue
                            if _alias==name:
                                continue
                            _id = get_document_product_dict_standard_alias_id(_alias)
                            data = [[_id],
                                    [_alias],
                                    [name],
                                    [id],
                                    [vector],
                                    [parent_id],
                                    [grade]]
                            insert_embedding(Coll,data)

                    _pd = Document_product_dict({DOCUMENT_PRODUCT_DICT_ID:id,DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED:2})
                    _pd.update_row(self.ots_client)


            except Exception as e:
                traceback.print_exc()


        self.embedding_producer()
        start_time = time.time()
        q_size = self.queue_product_dict.qsize()
        mt = MultiThreadHandler(self.queue_product_dict,handle,None,5,1)
        mt.run()
        log("process embedding %d records cost %.2f s"%(q_size,time.time()-start_time))


    def start_embedding_product_dict(self):
        from apscheduler.schedulers.blocking import BlockingScheduler
        scheduler = BlockingScheduler()
        # scheduler.add_job(func=self.embedding_producer,trigger="cron",minute="*/1")
        scheduler.add_job(func=self.embedding_comsumer,trigger="cron",second="*/5")
        scheduler.start()

    def delete_collections(self):
        drop_embedding_collection(self.collection_name_name)
        drop_embedding_collection(self.collection_name_brand)
        drop_embedding_collection(self.collection_name_specs)

def start_embedding_product_dict():
    pdm = Product_Dict_Manager()
    pdm.start_embedding_product_dict()

def drop_product_dict_collections():
    pdm = Product_Dict_Manager()
    pdm.delete_collections()


def search_similar():

    task_queue = Queue()
    ots_client = getConnect_ots()
    pdm = Product_Dict_Manager()

    list_data = []

    columns=[DOCUMENT_PRODUCT_DICT_NAME,DOCUMENT_PRODUCT_DICT_PARENT_ID,DOCUMENT_PRODUCT_DICT_GRADE]

    bool_query = BoolQuery(
        must_queries=[RangeQuery(DOCUMENT_PRODUCT_DICT_GRADE,5,5,True,True)]
    )

    rows,next_token,total_count,is_all_succeed = ots_client.search("document_product_dict","document_product_dict_index",
                                                                        SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED)]),limit=100,get_total_count=True),
                                                                        columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))

    list_dict = getRow_ots(rows)
    for _d in list_dict:
        list_data.append(_d)

    while next_token:
        rows,next_token,total_count,is_all_succeed = ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))
        list_dict = getRow_ots(rows)
        for _d in list_dict:
            list_data.append(_d)
        if len(list_data)>=100000:
            break
    log("product_dict embedding total_count:%d"%total_count)
    for _d in list_data:
        task_queue.put(_d)

    result_queue = Queue()

    def handle(item,result_queue):
        id = item.get(DOCUMENT_PRODUCT_DICT_ID)
        name = item.get(DOCUMENT_PRODUCT_DICT_NAME)
        vector = pdm.request_embedding(name)
        parent_id = item.get(DOCUMENT_PRODUCT_DICT_PARENT_ID)
        grade = item.get(DOCUMENT_PRODUCT_DICT_GRADE)
        Coll,Coll_name = pdm.get_collection(grade)
        output_fields = ['ots_id','ots_name',"ots_parent_id"]
        if vector is not None and Coll is not None:
            search_list = search_embedding(Coll,embedding_index_name,[vector],pdm.search_params,output_fields,limit=10)
            for _item in search_list:
                ots_id = _item.id
                ots_name = _item.entity.get("ots_name")
                ots_parent_id = _item.entity.get("ots_parent_id")
                if name!=ots_name:
                    if is_similar(name,ots_name):
                        _d = {"source_id":id,"source_name":name,"grade":grade,"target_id":ots_id,"target_name":ots_name,"parent_id":parent_id,"target_parent_id":ots_parent_id}
                        result_queue.put(_d)


    mt = MultiThreadHandler(task_queue,handle,result_queue,5,1)
    mt.run()

    df_data = {}
    _set = set()
    df_columns = ["source_id","source_name","grade","parent_id","target_id","target_name","parent_parent_id"]
    while 1:
        try:
            item = result_queue.get(timeout=1)
            source_name = item.get("source_name")
            target_name = item.get("target_name")
            _key1 = "%s-%s"%(source_name,target_name)
            _key2 = "%s-%s"%(target_name,source_name)
            for c in df_columns:
                if c not in df_data:
                    df_data[c] = []
                df_data[c].append(getLegal_str(item.get(c)))

        except Exception as e:
            break
    import pandas as pd
    df = pd.DataFrame(df_data)
    df.to_excel("search_similar1.xlsx",columns=df_columns)




if __name__ == '__main__':
    start_embedding_product_dict
